#import "HoppingViewController.h"
#import "Utils.h"
#import <ASRP31N_SDK/ASRP31N_SDK.h>

@interface HoppingViewController ()<ASRP31NSDKDelegate>
{
    Boolean isReceivedFhInfo;
    int16_t readTime ;
    int16_t idleTime ;
    int16_t cst;
    int16_t rfl;
    int16_t fh;
    int16_t lbt;
    int16_t cw;
}

@property (weak, nonatomic) IBOutlet UILabel *labelInfo;
@property (weak, nonatomic) IBOutlet UISwitch *swOnOff;
@property (weak, nonatomic) IBOutlet UISwitch *swMode;
@property (weak, nonatomic) IBOutlet UIButton *btnStep2;

-(IBAction)actionSwitchHoppingOnOff:(UISwitch *)sender;
-(IBAction)btnStep1:(id)sender;
-(IBAction)btnStep2:(id)sender;
-(IBAction)btnRefleshInfo:(id)sender;
-(IBAction)btnSelectHoppingmode:(id)sender;
@end

@implementation HoppingViewController

//----------------------------------------------
#pragma mark - Lifecycle
//----------------------------------------------
- (void)viewDidLoad
{
    [super viewDidLoad];
    self.title = @"Hopping";
    [self getInfo];
}

- (void)viewWillAppear:(BOOL)animated
{
    [ASRP31NSDK sharedInstance].delegate = self;
    /*20190621  Response Time Out*/
    [Utils showLoadingView:@"Check Hopping" timeOut:df_TIMEOUT_DEFAULT];
  
    [self performSelector:@selector(onTickGetFHInfo)     withObject:nil afterDelay:0.2f];
    [self performSelector:@selector(onTickSmartHoppingInfo)     withObject:nil afterDelay:0.4f];
    
    [self.swMode setHidden:NO];
    [self.btnStep2 setHidden:NO];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [ASRP31NSDK sharedInstance].delegate = nil;
}

//----------------------------------------------
#pragma mark - PrivateMethod
//----------------------------------------------
- (void)getInfo
{
    NSMutableString *strInfo = [[NSMutableString alloc]init];
    
    if(fh>lbt)
    {
        
        dispatch_async(dispatch_get_main_queue(),^{
            [strInfo appendString:@" FH ON/OFF : ON"];
            [self.swOnOff setOn:YES];
        });
      
    }
    else
    {
        dispatch_async(dispatch_get_main_queue(),^{
            [strInfo appendString:@" FH ON/OFF : OFF"];
            [self.swOnOff setOn:NO];
        });
    }
    
    [strInfo appendString:[NSString stringWithFormat:@"\n %@:%d",@"On Time : ",readTime]];
    [strInfo appendString:[NSString stringWithFormat:@"\n %@:%d",@"Off Time : ",idleTime]];
    [strInfo appendString:[NSString stringWithFormat:@"\n %@:%d",@"CST : ",cst]];
    [strInfo appendString:[NSString stringWithFormat:@"\n %@:%d",@"FH Val : ",fh]];
    [strInfo appendString:[NSString stringWithFormat:@"\n %@:%d",@"LBT : ",lbt]];
    [strInfo appendString:[NSString stringWithFormat:@"\n %@:%d",@"RFL : ",rfl]];
    [strInfo appendString:[NSString stringWithFormat:@"\n %@:%d",@"CW  : ",cw]];
    
    dispatch_async(dispatch_get_main_queue(),^{
        [self.labelInfo setText:strInfo];
    });
    /*20190621  Response Time Out*/
   [Utils hiddenLoadingView];
}

- (void)onTickGetFHInfo
{
    [[ASRP31NSDK sharedInstance] getFHLBTParam];
}

- (void)onTickSmartHoppingInfo
{
    [[ASRP31NSDK sharedInstance] getSmartHoppingOnOff];
}

//----------------------------------------------
#pragma mark - IBAction
//----------------------------------------------
- (IBAction)btnSelectHoppingmode:(id)sender
{
    UISwitch *sw = (UISwitch*)sender;
    if(sw.isOn)
        [Utils showAlert:@"Confirm" message:@"Selected Smart Mode   => push Step2" view:self];
    else
        [Utils showAlert:@"Confirm" message:@"Selected Normal Mode  => push Step2" view:self];
}

- (IBAction)actionSwitchHoppingOnOff:(UISwitch *)sender
{
    if(sender.isOn == YES) {
        [[ASRP31NSDK sharedInstance]setFHLBTParamRFLevel:rfl
                                                     readTime:readTime
                                                     idleTime:idleTime
                                             carrierSenseTime:cst
                                             frequencyHopping:1
                                             listenBeforeTalk:0
                                               continuousWave:0];
        
    } else {
        [[ASRP31NSDK sharedInstance]setFHLBTParamRFLevel:rfl
                                                     readTime:readTime
                                                     idleTime:idleTime
                                             carrierSenseTime:cst
                                             frequencyHopping:0
                                             listenBeforeTalk:1
                                               continuousWave:0];
    }
}

/* Reflesh information of Hopping data*/
- (IBAction)btnRefleshInfo:(id)sender
{
     [[ASRP31NSDK sharedInstance] getFHLBTParam];
}


- (IBAction)btnStep1:(id)sender
{
    [[ASRP31NSDK sharedInstance] setOptimumFrequencyHoppingTable];
}


- (IBAction)btnStep2:(id)sender
{
    [[ASRP31NSDK sharedInstance] setSmartHoppingOnOff:self.swMode.on];
}

//----------------------------------------------
#pragma mark - ASRP31NSDK Delegate
//----------------------------------------------
- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedFHLBT:(NSData *)fhlb;
{
    Byte *b = (Byte*) [fhlb bytes];
    if (!(fhlb.length < 11)) {
        readTime = (b[0] << 8) | b[1];
        idleTime = (b[2] << 8) | b[3];
        cst = (b[4] << 8) | b[5];
        rfl = (b[6] << 8) | b[7];
        fh  = b[8];
        lbt = b[9];
        cw = b[10];
    }
    [self getInfo];
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedDidSetFHLBT:(ASRP31NSDKCommonStatus)statusCode
{
    [Utils showAlert:@"Confirm" message:@"Set FH/LBT Param" view:self];
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedDidSetOptiFreqHPTable:(uint8_t)statusCode
{
    if(statusCode == 0x00) {
        /*20190621  Response Time Out*/
        [Utils showLoadingView:@"Wait!! Start OptiMum Freq Start" timeOut:df_TIMEOUT_DEFAULT];
    } else if(statusCode == 0x01){
        /*20190621  Response Time Out*/
       [Utils hiddenLoadingView];
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedDidSetSmartHopping:(uint8_t)statusCode
{
    switch (statusCode) {
        case ASRP31NSDKCommonStatusSuccess:
            [Utils showAlert:@"Success" message:@"Changed Smart Hopping Mode" view:self];
            break;
            
        case ASRP31NSDKCommonStatusFailure:
            [Utils showAlert:@"Fail" message:@"Changed Smart Hopping Mode" view:self];
            break;
        default:
            break;
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedSmartHopping:(BOOL)isON
{
    dispatch_async(dispatch_get_main_queue(),^{
        [self.swMode setOn:isON];
    });
    
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedErrDetail:(NSData *)errCode
{
    Byte *b = (Byte*) [errCode bytes];
    /*20190621  Response Time Out*/
   [Utils hiddenLoadingView];
    dispatch_async(dispatch_get_main_queue(),^{
        switch (b[1])
        {
            case 0xE4:
            case 0xE6:
            case 0xD2:
            {
                [Utils showAlert:@"Error" message:@"The retrun loss of antenna is too large to optimize channel." view:self];
                break;
            }
            default:
                
                break;
        }
    });
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N changeNetworkState:(ASRP31NSDKNetworkState)state error:(NSError *)error
{
    switch (state) {
            
        case ASRP31NSDKNetworkStateDisconnected:
            [Utils showAlert:@"Fail" message:@"The network connection is lost." view:self];
            break;
            
        case ASRP31NSDKNetworkStateError:
            break;
            
        default:
            break;
    }
}


// ↓ これは必要ですか？ Y.Oshiro 2019/04/17
//- (void)didSetFHmodeChanged
//{
//    [Utils showAlert:@"Confirm" message:@"didSetFhModeChange" view:self];
//
//}
@end
