#import "NetworkSettingController.h"
#import "Utils.h"
#import <ASRP31N_SDK/ASRP31N_SDK.h>

@interface NetworkSettingController ()<ASRP31NSDKDelegate>
{
    NSMutableArray *arryDeviceInfo;
    
}
@property (weak, nonatomic) IBOutlet UIView *viewStatic;
@property (weak, nonatomic) IBOutlet UITextField *txtFieldIP;
@property (weak, nonatomic) IBOutlet UITextField *txtFieldSubnet;
@property (weak, nonatomic) IBOutlet UITextField *txtFieldGateWay;
@property (weak, nonatomic) IBOutlet UITextField *txtFieldDnsServer;
@property (weak, nonatomic) IBOutlet UITextField *txtFieldMacAddr;
@property (weak, nonatomic) IBOutlet UISegmentedControl *segmentDHCPStatic;

-(IBAction)searchButtonAction:(id)sender;
-(IBAction)doneButtonAction:(id)sender;
-(IBAction)segmentButtonAction:(id)sender;
@end

@implementation NetworkSettingController

//----------------------------------------------
#pragma mark - Lifecycle
//----------------------------------------------
-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    [ASRP31NSDK sharedInstance].delegate = self;
    [self searchDevice];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view.
}

//----------------------------------------------
#pragma mark - IBAction
//----------------------------------------------
-(IBAction)searchButtonAction:(id)sender
{
    [self searchDevice];
}

-(IBAction)doneButtonAction:(id)sender
{
    int selectIndex  =  (int)[self.segmentDHCPStatic selectedSegmentIndex];
    
    NSString *mac = self.txtFieldMacAddr.text;
    NSString *ip = self.txtFieldIP.text;
    NSString *sub = self.txtFieldSubnet.text;
    NSString *gw  = self.txtFieldGateWay.text;
    NSString *dns = self.txtFieldDnsServer.text;
    
    if(selectIndex == 0) {
        //DHCP
        [[ASRP31NSDK sharedInstance] setDHCPOnMac:mac];
    } else if(selectIndex == 1) {
        //Static IP
        [[ASRP31NSDK sharedInstance] setDHCPOffMac:mac IP:ip Subnet:sub GateWay:gw DNSServer:dns];
    }
    
    [Utils showAlert:@"Confirm" message:@"The network has changed.Press the [Search again] button and check the changed information." view:self];
}

-(IBAction)segmentButtonAction:(id)sender
{
    int selectIndex  =  (int)[self.segmentDHCPStatic selectedSegmentIndex];
    //DHCP
    if(selectIndex == 0) {
        [self enableTextField:NO];
    }
    
    //Static IP
    else if(selectIndex == 1){
        [self enableTextField:YES];
    }
}

//----------------------------------------------
#pragma mark - PrivateMethod
//----------------------------------------------
- (void)onTickHiddenAlert
{
    /*20190621  Response Time Out*/
    [Utils hiddenLoadingView];
}

- (void)searchDevice
{
    [[ASRP31NSDK sharedInstance]startSearchDevice];
    /*20190621  Response Time Out*/
    [Utils showLoadingView:@"Searching Device" timeOut:df_TIMEOUT_NONE];
    [self performSelector:@selector(onTickHiddenAlert) withObject:nil afterDelay:5.0f];
}

- (void)enableTextField:(BOOL)isOn
{
    [self.txtFieldMacAddr setEnabled:isOn];
    [self.txtFieldIP setEnabled:isOn];
    [self.txtFieldSubnet setEnabled:isOn];
    [self.txtFieldGateWay setEnabled:isOn];
    [self.txtFieldDnsServer setEnabled:isOn];
}

- (void)showAlertViewAllDevics
{
    UIAlertController* alert = [UIAlertController alertControllerWithTitle:@"[Found Device] For DHCP "
                                                                   message:@"Please select device IP"
                                                            preferredStyle:UIAlertControllerStyleActionSheet];
    
    for(int i=0; i<[arryDeviceInfo count]; i++)
    {
        NSDictionary *device = [arryDeviceInfo objectAtIndex:i];
        
        NSString *sIp = [device objectForKey:@"ip"];
        
        
        UIAlertAction* ip = [UIAlertAction actionWithTitle:sIp
                                                     style:UIAlertActionStyleDefault
                                                   handler:^(UIAlertAction * action) {
                                                       NSDictionary *device = [self->arryDeviceInfo objectAtIndex:i];
                                                       [self.txtFieldIP setText: [device objectForKey:@"ip"]];
                                                       [self.txtFieldSubnet setText:[device objectForKey:@"subnet"]];
                                                       [self.txtFieldGateWay setText:[device objectForKey:@"gateway"]];
                                                       [self.txtFieldDnsServer setText:[device objectForKey:@"dns"]];
                                                       [self.txtFieldMacAddr setText:[device objectForKey:@"mac"]];
                                                       NSString *mode =[device objectForKey:@"dhcp"];
                                                       if([mode isEqualToString:@"DHCP"])
                                                       {
                                                           [self.segmentDHCPStatic setSelectedSegmentIndex:0];
                                                           [self enableTextField:NO];
                                                       }
                                                       else
                                                       {
                                                           [self.segmentDHCPStatic setSelectedSegmentIndex:1];
                                                           [self enableTextField:YES];
                                                       }
//                                                       NSLog(@" %@",device.sGateWay);
                                                   }];
        [alert addAction:ip];
    }
    
    UIAlertAction* cancel = [UIAlertAction actionWithTitle:@"Cancel"
                                                     style:UIAlertActionStyleDefault
                                                   handler:nil];
    [alert addAction:cancel];
    
    UIBarButtonItem *button;
    alert.popoverPresentationController.barButtonItem = button;
    alert.popoverPresentationController.sourceView = self.view;
    [self presentViewController:alert animated:YES completion:nil];
}

//----------------------------------------------
#pragma mark - ASRP31NSDK Delegate
//----------------------------------------------
- (void)ASRP31N:(ASRP31NSDK *)ASRP31N searchedUPDSearchingDeviceInfo:(NSArray*)arrInfos;
{
   /*20190621  Response Time Out*/
    [Utils hiddenLoadingView];
    [arryDeviceInfo removeAllObjects];
    arryDeviceInfo = [[NSMutableArray alloc] initWithArray:arrInfos];
    [self showAlertViewAllDevics];
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N searchingIP:(NSString *)ip isFinish:(BOOL)isFinish
{
    if(isFinish == YES) { //Finsh Search Device
        /*20190621  Response Time Out*/
       [Utils hiddenLoadingView];
    }
}

@end
