#import "RFIDTvController.h"
#import <ASRP31N_SDK/ASRP31N_SDK.h>
#import "Utils.h"


@interface RFIDTvController ()<ASRP31NSDKDelegate>
{

}

@property (weak, nonatomic) IBOutlet UILabel   *labelOnOffTime;
@property (weak, nonatomic) IBOutlet UILabel   *labelSession;
@property (weak, nonatomic) IBOutlet UILabel   *labelChannel;
@property (weak, nonatomic) IBOutlet UILabel   *labelOuputPower;
@property (weak, nonatomic) IBOutlet UISwitch  *swRSSI;
-(IBAction)btnUpdate:(id)sender;
-(IBAction)btnRSSIOnOff:(id)sender;
@property (weak, nonatomic) IBOutlet UISegmentedControl *operationModeSegment;
@end


@implementation RFIDTvController

//----------------------------------------------
#pragma mark - Lifecycle
//----------------------------------------------
- (void)viewDidLoad
{
    [super viewDidLoad];   
}

-(IBAction)btnRSSIOnOff:(id)sender
{
    [[NSUserDefaults standardUserDefaults] setBool:_swRSSI.isOn forKey:@"RSSI"];
    [Utils showAlert:@"Confirm" message:@"RSSI On/Off Changed" view:self];
}

- (void)viewWillAppear:(BOOL)animated
{
    [ASRP31NSDK sharedInstance].delegate = self;
    /*20190621  Response Time Out*/
     [Utils showLoadingView:@"Check Settings" timeOut:df_TIMEOUT_DEFAULT];
    
    BOOL isRSSIOn    = (BOOL)[[NSUserDefaults standardUserDefaults] boolForKey:@"RSSI"];
    [self.swRSSI setOn:isRSSIOn];
    
    BOOL isOperationMode = (BOOL)[[NSUserDefaults standardUserDefaults] boolForKey:@"OperationMode"];
    [self.operationModeSegment setSelectedSegmentIndex:isOperationMode];
    
    //OnSession
    [self performSelector:@selector(onTickGetSession)     withObject:nil afterDelay:.2f];
    
    //Channel
    [self performSelector:@selector(onTickGetChannel)     withObject:nil afterDelay:.4f];
    
}

-(void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:YES];
    [ASRP31NSDK sharedInstance].delegate = nil;
}

//----------------------------------------------
#pragma mark - PrivateMethod
//----------------------------------------------

-(void) onTickGetSession
{
    NSError *error = [[ASRP31NSDK sharedInstance] getSession];
    if( error ){
        NSLog(@"%@\n code = %ld\n%@", NSStringFromSelector(_cmd), error.code, error.userInfo);
    }
   
}

-(void) onTickGetChannel
{
    NSError *error = [[ASRP31NSDK sharedInstance] getChannel];
    if( error ){
        NSLog(@"%@\n code = %ld\n%@", NSStringFromSelector(_cmd), error.code, error.userInfo);
    }
    /*20190621  Response Time Out*/
    [Utils hiddenLoadingView];
}

//----------------------------------------------
#pragma mark - IBAction
//----------------------------------------------
-(IBAction)btnUpdate:(id)sender
{
    NSError *error = [[ASRP31NSDK sharedInstance] updateRegistry];
    if( error ){
        NSLog(@"%@\n code = %ld\n%@", NSStringFromSelector(_cmd), error.code, error.userInfo);
    }
}

- (IBAction)onClickOperationModelSegment:(id)sender {
    [[NSUserDefaults standardUserDefaults] setBool:_operationModeSegment.selectedSegmentIndex forKey:@"OperationMode"];
    [Utils showAlert:@"Confirm" message:@"Operation Mode Changed" view:self];
}

//----------------------------------------------
#pragma mark - ASRP31NSDK Delegate
//----------------------------------------------
- (void)ASRP31N:(ASRP31NSDK *)ASRP31N updatedRegistryStatus:(uint8_t)statusCode
{
    if(statusCode == 0x00) {
        [Utils showAlert:@"Success" message:@"Update registry" view:self];
    } else{
        [Utils showAlert:@"Fail" message:@"Update registry" view:self];
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedSession:(uint8_t)session
{
    dispatch_async(dispatch_get_main_queue(),^{
        self->_labelSession.text = [NSString stringWithFormat:@"%d",session];
    });
}

-(void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedChannel:(uint8_t)channel offset:(uint8_t)offset
{
    dispatch_async(dispatch_get_main_queue(),^{
        self->_labelChannel.text = [NSString stringWithFormat:@"%d",channel];
    });
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N changeNetworkState:(ASRP31NSDKNetworkState)state error:(NSError *)error
{
    switch (state) {
            
        case ASRP31NSDKNetworkStateDisconnected:
            [Utils showAlert:@"Fail" message:@"The network connection is lost." view:self];
            break;
            
        case ASRP31NSDKNetworkStateError:
            break;
            
        default:
            break;
    }
}

@end
