//  ReadWriteController.m

#import "ReadWriteController.h"
#import "SelectedTagInfo.h"
#import "Utils.h"
#import <ASRP31N_SDK/ASRP31N_SDK.h>
#import "PhyUtility.h"
@interface ReadWriteController () <UITextFieldDelegate, ASRP31NSDKDelegate>
{

}
//@property (nonatomic, strong, readwrite) NSData *epc;
@property (weak, nonatomic) IBOutlet UISegmentedControl *olMode;
@property (weak, nonatomic) IBOutlet UITextField *olAccessPassword;
@property (weak, nonatomic) IBOutlet UITextField *olStartAdress;
@property (weak, nonatomic) IBOutlet UITextField *olLength;
@property (weak, nonatomic) IBOutlet UITextField *olData;
@property (weak, nonatomic) IBOutlet UISegmentedControl *olTargetMemory;
@property (weak, nonatomic) IBOutlet UITextField *olTargetEpc;
@property (weak, nonatomic) IBOutlet UILabel *olDataTitle;
@property (weak, nonatomic) IBOutlet UIScrollView *olScrollVeiw;

- (IBAction)rightBarButtonItemClicked:(id)sender;
- (IBAction)eTargetMemoryChanged:(id)sender;
- (IBAction)eModeChanged:(id)sender;
- (IBAction)actLimitEPCLength:(id)sender;
@end

@implementation ReadWriteController
//----------------------------------------------
#pragma mark - Lifecycle
//----------------------------------------------
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    SelectedTagInfo *selectTag =  [SelectedTagInfo sharedInstance];
    self.olTargetEpc.text =   selectTag.selectedTagEPC;
    
    [ASRP31NSDK sharedInstance].delegate = self;
    self.olAccessPassword.delegate  = self;
    self.olStartAdress.delegate     = self;
    self.olLength.delegate          = self;
    self.olData.delegate            = self;
    self.olTargetEpc.delegate       = self;
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [ASRP31NSDK sharedInstance].delegate = nil;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.olScrollVeiw.scrollEnabled = YES;
    self.olScrollVeiw.contentSize = CGSizeMake(self.view.frame.size.width, self.view.frame.size.height + 600);
    [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:self.view.window];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//----------------------------------------------
#pragma mark - PrivateMethod
//----------------------------------------------
- (NSString *) dataToEpcInteger:(NSData *)data
{
    uint8_t* pData = (uint8_t*)[data bytes];
    uint16_t intData = pData[0] << 8 | pData[1];
    intData = intData >>11;
    return [NSString stringWithFormat:@"%d",intData];
}

-(void) setViewMoveUp:(BOOL)moveUp
{
    if(moveUp) {
        [UIView animateWithDuration:0.2 animations:^{self.olScrollVeiw.contentOffset = CGPointMake(0.0f, 100.f);}];
    } else {
        [UIView animateWithDuration:0.2 animations:^{self.olScrollVeiw.contentOffset = CGPointMake(0.0f, self.olScrollVeiw.frame.origin.y-80);}];
    }
}

- (void)keyboardWillShow:(NSNotification *)notif
{
    //olData
    if([self.olData isFirstResponder] &&self.view.frame.origin.y >=0) {
        [self setViewMoveUp:YES];
    } else if ([self.olData isFirstResponder]&& self.view.frame.origin.y <0) {
        [self setViewMoveUp:NO];
    }
}

//----------------------------------------------
#pragma mark - UITextField Delegate
//----------------------------------------------
- (BOOL)textFieldShouldReturn:(UITextField *)txtField
{
    [self setViewMoveUp:NO];
    [txtField resignFirstResponder];
    return NO;
}

//----------------------------------------------
#pragma mark - IBAction
//----------------------------------------------
- (IBAction)eModeChanged:(id)sender
{
    if(self.olMode.selectedSegmentIndex == 0) {
        self.navigationItem.title = @"Read";
    } else {
        self.navigationItem.title = @"Write";
    }
}

- (IBAction)rightBarButtonItemClicked:(id)sender
{
    [self.olTargetEpc resignFirstResponder];
    [self.olAccessPassword resignFirstResponder];
    [self.olStartAdress resignFirstResponder];
    [self.olLength resignFirstResponder];
    [self.olData resignFirstResponder];
    [self setViewMoveUp:NO];
    NSString  *hexaString = self.olAccessPassword.text;
    NSScanner *pScanner = [NSScanner scannerWithString: hexaString];
    uint32_t  accesspassword;
    [pScanner scanHexInt: &accesspassword];
    
    uint16_t  startAddress = [self.olStartAdress.text intValue];
    uint16_t  dataLength = [self.olLength.text intValue];
    uint8_t   membank =0;
    
    switch(self.olTargetMemory.selectedSegmentIndex) {
        case 0:
            membank = 0;
            break;
        case 1: //PC
        case 2: //EPC
            membank = 1;
            break;
        case 3: // TID
            membank = 2;
            break;
        case 4:
            membank = 3;
            break;
    }
    
    NSData *data     = [self.olData.text hexStringToBytes];
    NSData *epcs     = [self.olTargetEpc.text hexStringToBytes];
    NSData *pcData   = [epcs subdataWithRange:NSMakeRange(0, 2)];
    
    if(epcs.length == 0) return;
    
    if(self.olMode.selectedSegmentIndex == 0) {
        [[ASRP31NSDK sharedInstance] readFromTagMemory:accesspassword epc:epcs memoryBank:membank startAddress:startAddress dataLength:dataLength];
    } else {
        if (self.olTargetMemory.selectedSegmentIndex == 1) {
            uint8_t* pData = (uint8_t*)[pcData bytes];
            uint16_t intData = pData[0] << 8 | pData[1];
            int epcLength = [self.olData.text intValue];
            if (epcLength != 0) {
                intData = intData & 0x07FF;
                intData |= (epcLength& 0x1F) << 11;
            }
            data = [[NSString stringWithFormat:@"%04X",intData]hexStringToBytes];
        }
        
        //20190625 Write TAG memory Debug
        [[ASRP31NSDK sharedInstance]setWriteToTagMemory:accesspassword epc:epcs memoryBank:membank startAddress:startAddress dataToWrite:data];
    }
}

- (IBAction)eTargetMemoryChanged:(id)sender
{
    self.olDataTitle.text = @"Data (HEX)";
    [self.olData resignFirstResponder];
    [self.olData setKeyboardType:UIKeyboardTypeDefault];
    
    if (self.olTargetMemory.selectedSegmentIndex == 1) {
        //PC
        self.olStartAdress.text     = @"1";
        self.olLength.text          = @"1";
        self.olStartAdress.enabled  = NO;
        self.olLength.enabled       = NO;
        self.olDataTitle.text       = @"EPC Length";
        [self.olData setKeyboardType:UIKeyboardTypeNumberPad];
    } else if(self.olTargetMemory.selectedSegmentIndex == 2) {
        //EPC
        self.olStartAdress.text     = @"2";
        self.olLength.text          = @"0";
        self.olStartAdress.enabled  = NO;
        self.olLength.enabled       = NO;
    } else {
        self.olStartAdress.enabled  = YES;
        self.olLength.enabled       = YES;
    }
}

//----------------------------------------------
#pragma mark - ASRP31NDelegate
//----------------------------------------------
- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedTagMemory:(NSData *)data
{
    dispatch_async(dispatch_get_main_queue(),^{
        NSString *strData;
        if (self.olTargetMemory.selectedSegmentIndex == 1) {
            strData = [self dataToEpcInteger:data];
        } else {
            strData = [data bytesToHexString];
        }
        self.olData.text = strData;
    });
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedWritedStatus:(ASRP31NSDKCommonStatus)statusCode
{
    switch (statusCode) {
        case ASRP31NSDKCommonStatusSuccess:
            [Utils showAlert:@"Success" message:@"Writed" view:self];
            break;
            
        case ASRP31NSDKCommonStatusFailure:
            [Utils showAlert:@"Fail" message:@"Writed" view:self];
            break;
        default:
            break;
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedErrDetail:(NSData *)errCode
{
    NSString *errMessage = [NSString stringWithFormat:@"%@",errCode];
    [Utils showAlert:@"Err code" message:errMessage view:self];
}


- (IBAction)actLimitEPCLength:(id)sender {
    if (self.olTargetMemory.selectedSegmentIndex == 1)
    {
        // 31까지 숫자 제한 : 5bit
        if ([self.olData.text intValue] > 31) {
            self.olData.text = @"31";
        }
    }
}


- (void)ASRP31N:(ASRP31NSDK *)ASRP31N changeNetworkState:(ASRP31NSDKNetworkState)state error:(NSError *)error
{
    switch (state) {
            
        case ASRP31NSDKNetworkStateDisconnected:
            [Utils showAlert:@"Fail" message:@"The network connection is lost." view:self];
            break;
            
        case ASRP31NSDKNetworkStateError:
            break;
            
        default:
            break;
    }
}

@end



