#import "SessionController.h"
#import "Utils.h"
#import <ASRP31N_SDK/ASRP31N_SDK.h>

@interface SessionController ()<ASRP31NSDKDelegate>
{
    int selectedSession;
}

@end

@implementation SessionController

//----------------------------------------------
#pragma mark - Lifecycle
//----------------------------------------------
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [ASRP31NSDK sharedInstance].delegate = self;
    
    [self performSelector:@selector(onTickGetSession)   withObject:nil afterDelay:.4f];
    /*20190621  Response Time Out*/
    [Utils showLoadingView:@"Searching Device" timeOut:df_TIMEOUT_DEFAULT];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [ASRP31NSDK sharedInstance].delegate = nil;
}

//----------------------------------------------
#pragma mark - IBAction
//----------------------------------------------
- (IBAction)btnSetSession:(id)sender
{
    NSError *error = [[ASRP31NSDK sharedInstance] setSession:selectedSession];
    if( error ){
        NSLog(@"%@\n code = %ld\n%@", NSStringFromSelector(_cmd), (int)error.code, error.userInfo);
    }
}

//----------------------------------------------
#pragma mark - Private Method
//----------------------------------------------
- (void)onTickGetSession
{
    NSError *error = [[ASRP31NSDK sharedInstance] getSession];
    if( error ){
        NSLog(@"%@\n code = %ld\n%@", NSStringFromSelector(_cmd), error.code, error.userInfo);
    }
    
   /*20190621  Response Time Out*/
   [Utils hiddenLoadingView];
}

//----------------------------------------------
#pragma mark - ASRP31NSDK Delegate
//----------------------------------------------
- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedDidSetSessionState:(ASRP31NSDKCommonStatus)statusCode
{
    switch (statusCode) {
        case ASRP31NSDKCommonStatusSuccess:
             [Utils showAlert:@"Success" message:@"Session changed" view:self];
            break;
        
        case ASRP31NSDKCommonStatusFailure:
            [Utils showAlert:@"Fail" message:@"Session changed" view:self];
            break;
            
        default:
            break;
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedSession:(uint8_t)session
{
    /*20190621  Response Time Out*/
    [Utils hiddenLoadingView];
    selectedSession = session;
    dispatch_async(dispatch_get_main_queue(),^{
        [self.tableView reloadData];
    });
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N changeNetworkState:(ASRP31NSDKNetworkState)state error:(NSError *)error
{
    switch (state) {
            
        case ASRP31NSDKNetworkStateDisconnected:
            [Utils showAlert:@"Fail" message:@"The network connection is lost." view:self];
            break;
            
        case ASRP31NSDKNetworkStateError:
            break;
            
        default:
            break;
    }
}


//----------------------------------------------
#pragma mark - TableView DataSource
//----------------------------------------------
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return 4;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    cell.textLabel.text = [NSString stringWithFormat:@"S%d", (int)indexPath.row];
    
    if(selectedSession == indexPath.row)
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    else
        cell.accessoryType = UITableViewCellAccessoryNone;
    return cell;
}

//----------------------------------------------
#pragma mark - TableView Delegate
//----------------------------------------------
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    selectedSession = (int)indexPath.row;
    [self.tableView reloadData];
}
@end
