#import "ViewController.h"
#import <ASRP31N_SDK/ASRP31N_SDK.h>
#import "Utils.h"
#import "TagData.h"
#import "TagViewController.h"

@interface ViewController ()<ASRP31NSDKDelegate>
{
    NSMutableArray *arrIPs;
    NSString *selectIP;
    TagData *tagData;
    SystemSoundID soundFileObject;
}

@property (nonatomic,retain) IBOutlet UIBarButtonItem *btnMore;
@property (nonatomic,retain) IBOutlet UIButton *btnRead;
@property (nonatomic,retain) IBOutlet UILabel *labelIP;
@property (nonatomic,retain) IBOutlet UILabel *labelTags;
@property (nonatomic,retain) IBOutlet UIView *viewBottom;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt1;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt2;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt3;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt4;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt5;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt6;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt7;
@property (nonatomic,retain) IBOutlet UIButton *btnCheckAnt8;
@property (nonatomic, strong) TagViewController            *tagViewController;

- (IBAction)searchDeviceButtonAction:(id)sender;
- (IBAction)checkButtonAction:(id)sender;
- (IBAction)readButtonAction:(id)sender;
- (IBAction)stopButtonAction:(id)sender;
- (IBAction)clearButtonAction:(id)sender;

@end


@implementation ViewController

//----------------------------------------------
#pragma mark - Lifecycle
//----------------------------------------------
- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.btnMore setEnabled:NO];
    [self setUpSound];
    
  
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    tagData = [TagData sharedInstance];
    if(arrIPs == nil)
        arrIPs  = [[NSMutableArray alloc]init];
    
    [ASRP31NSDK sharedInstance].delegate = self;
    
    NSString* version = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
    self.title =  [NSString stringWithFormat:@"AsReader(A:%@/S:%@)",version, [[ASRP31NSDK sharedInstance] getSDKVersion]];
    
    if(![[ASRP31NSDK sharedInstance]getConnectionStatus])
      [self enableUis:NO];
    
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [ASRP31NSDK sharedInstance].delegate = nil;
}

- (void)viewDidDisappear:(BOOL)animated
{
    [self stopButtonAction:nil];
}

//----------------------------------------------
#pragma mark - Segue
//----------------------------------------------
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"segueTagTableView"]) {
        self.tagViewController = segue.destinationViewController;
    }
}

//----------------------------------------------
#pragma mark - IBAction
//----------------------------------------------
- (IBAction)searchDeviceButtonAction:(id)sender
{
    /*20190621  Response Time Out*/
    [Utils showLoadingView:@"Searching" timeOut:df_TIMEOUT_DEFAULT+2];

    [arrIPs removeAllObjects];
    [[ASRP31NSDK sharedInstance] startSearchDevice];
}

- (IBAction)checkButtonAction:(id)sender
{
    UIButton *btn = (UIButton*)sender;
    
    if(btn.tag == 0) { //Check
        btn.tag = 1;
        [btn setImage:[UIImage imageNamed:@"checked.png"] forState:UIControlStateNormal];
    } else {    //UnCheck
        btn.tag = 0;
        [btn setImage:[UIImage imageNamed:@"uncheck.png"] forState:UIControlStateNormal];
    }
}

- (IBAction)readButtonAction:(id)sender
{
    int nTagCount = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"RFIDScanTagCount"];
    int nScanTime = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"RFIDScanTagTime"];
    int nCycle    = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"RFIDScanTagInventory"];
    BOOL isRSSIOn    = (BOOL)[[NSUserDefaults standardUserDefaults] boolForKey:@"RSSI"];
    BOOL isOperationMode = (BOOL)[[NSUserDefaults standardUserDefaults] boolForKey:@"OperationMode"];
    
    if((_btnCheckAnt1.tag)||
       (_btnCheckAnt2.tag)||
       (_btnCheckAnt3.tag)||
       (_btnCheckAnt4.tag)||
       (_btnCheckAnt5.tag)||
       (_btnCheckAnt6.tag)||
       (_btnCheckAnt7.tag)||
       (_btnCheckAnt8.tag))
    {
    [[ASRP31NSDK sharedInstance] startMultiReadTags:nTagCount mtime:nScanTime repeatCycle:nCycle
                                     Antenna1Enable:(BOOL)_btnCheckAnt1.tag
                                     Antenna2Enable:(BOOL)_btnCheckAnt2.tag
                                     Antenna3Enable:(BOOL)_btnCheckAnt3.tag
                                     Antenna4Enable:(BOOL)_btnCheckAnt4.tag
                                     Antenna5Enable:(BOOL)_btnCheckAnt5.tag
                                     Antenna6Enable:(BOOL)_btnCheckAnt6.tag
                                     Antenna7Enable:(BOOL)_btnCheckAnt7.tag
                                     Antenna8Enable:(BOOL)_btnCheckAnt8.tag
                                         RSSIEnable:isRSSIOn operationMode:isOperationMode];
    }
    else
    {
        [Utils showAlert:@"Confirm" message:@"Please select Antenna" view:self];
        return;
    }
    
    [_btnRead setEnabled:NO];
    [_btnCheckAnt1 setEnabled:NO];
    [_btnCheckAnt2 setEnabled:NO];
    [_btnCheckAnt3 setEnabled:NO];
    [_btnCheckAnt4 setEnabled:NO];
    [_btnCheckAnt5 setEnabled:NO];
    [_btnCheckAnt6 setEnabled:NO];
    [_btnCheckAnt7 setEnabled:NO];
    [_btnCheckAnt8 setEnabled:NO];

}

- (IBAction)stopButtonAction:(id)sender
{
    [_btnRead setEnabled:YES];
    [_btnCheckAnt1 setEnabled:YES];
    [_btnCheckAnt2 setEnabled:YES];
    [_btnCheckAnt3 setEnabled:YES];
    [_btnCheckAnt4 setEnabled:YES];
    [_btnCheckAnt5 setEnabled:YES];
    [_btnCheckAnt6 setEnabled:YES];
    [_btnCheckAnt7 setEnabled:YES];
    [_btnCheckAnt8 setEnabled:YES];
    
    [[ASRP31NSDK sharedInstance] stopReadTags];
}

- (IBAction)clearButtonAction:(id)sender
{
    [tagData.arrTags removeAllObjects];
    [self.labelTags setText:[NSString stringWithFormat:@"%d Tags",(int)[tagData.arrTags count]]];
    [self.tagViewController updateData];
}

//----------------------------------------------
#pragma mark - ASRP31NSDK Delegate
//----------------------------------------------
- (void)ASRP31N:(ASRP31NSDK *)ASRP31N searchingIP:(NSString *)ip isFinish:(BOOL)isFinish
{
    if(isFinish == YES) { //Finsh Search Device
       
        /*20190621  Response Time Out*/
        [Utils hiddenLoadingView];
        
        if([arrIPs count]>0)
            [self showAlertViewIPs];
        else
            [Utils showAlert:@"Search" message:@"Not found Devices" view:self];
    } else {    //Searching....
        [arrIPs addObject:ip];
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N changeNetworkState:(ASRP31NSDKNetworkState)state error:(NSError *)error
{
    switch (state) {
        case ASRP31NSDKNetworkStateConnected:
            [self enableUis:YES];
            //[Utils showAlert:@"TCP" message:@"Disconnected" view:self];
            break;
            
        case ASRP31NSDKNetworkStateDisconnected:
            [self enableUis:NO];
            break;
            
        case ASRP31NSDKNetworkStateError:
            [self enableUis:NO];
            [Utils showAlert:@"Network" message:@"Fail connected" view:self];
            break;
            
        default:
            break;
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedPCEPCData:(NSData *)pcEpc selectAntena:(int)antenna
{
    NSMutableString* tag = [[NSMutableString alloc]init] ;
    unsigned char* ptr= (unsigned char*) [pcEpc bytes];
    
    for(int i = 0; i < pcEpc.length; i++)
        [tag appendFormat:@"%02x", *ptr++ & 0xFF ];
    
    [self addScanDataFiltering:tag rawData:pcEpc selectAnt:antenna RSSI:0 user:@""];
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedPCEPCData:(NSData *)pcEpc selectAntena:(int)antenna rssi:(int)rssiVal
{
    NSMutableString* tag = [[NSMutableString alloc]init] ;
    unsigned char* ptr= (unsigned char*) [pcEpc bytes];
    
    for(int i = 0; i < pcEpc.length; i++)
        [tag appendFormat:@"%02x", *ptr++ & 0xFF ];
    
    [self addScanDataFiltering:tag rawData:pcEpc selectAnt:antenna RSSI:rssiVal user:@""];
}



- (void)ASRP31N:(ASRP31NSDK *)ASRP31N readerConnectedStatus:(ASRP31NSDKCommonStatus)statusCode
{
    switch (statusCode) {
        case ASRP31NSDKCommonStatusSuccess:
            [Utils showAlert:@"Network Connected" message:@"RFID did power ON" view:self];
            [self enableUis:YES];
            break;
            
        case ASRP31NSDKCommonStatusFailure:
            [Utils showAlert:@"Error" message:@"Failue Network connected." view:self];
            [self enableUis:NO];
            break;
        default:
            break;
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N receivedStoppedReadTagsStatus:(ASRP31NSDKCommonStatus)statusCode
{
    NSLog(@"Stop Scan");
    switch (statusCode) {
        case ASRP31NSDKCommonStatusSuccess:
            break;
            
        case ASRP31NSDKCommonStatusFailure:
            
            break;
        default:
            break;
    }
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N pcEpcReceivedMultiAntmode:(NSData *)pcEpc selectAnt:(int)ant userData:(NSData *)userData
{
    NSMutableString* tag = [[NSMutableString alloc]init] ;
    unsigned char* ptr= (unsigned char*) [pcEpc bytes];
    for(int i = 0; i < pcEpc.length; i++){
        [tag appendFormat:@"%02x", *ptr++ & 0xFF ];
    }
    NSMutableString* userDataTag = [[NSMutableString alloc]init] ;
    unsigned char* ptrr= (unsigned char*) [userData bytes];
    for(int i = 0; i < userData.length; i++){
        [userDataTag appendFormat:@"%02x", *ptrr++ & 0xFF ];
    }
    [self addScanDataFiltering:tag rawData:pcEpc selectAnt:ant RSSI:0 user:userDataTag];
}

- (void)ASRP31N:(ASRP31NSDK *)ASRP31N pcEpcReceivedMultiAntmode:(NSData *)pcEpc selectAnt:(int)ant rssi:(int)rssi userData:(NSData *)userData
{
    NSMutableString* tag = [[NSMutableString alloc]init] ;
    unsigned char* ptr= (unsigned char*) [pcEpc bytes];
    for(int i = 0; i < pcEpc.length; i++){
        [tag appendFormat:@"%02x", *ptr++ & 0xFF ];
    }
    NSMutableString* userDataTag = [[NSMutableString alloc]init] ;
    unsigned char* ptrr= (unsigned char*) [userData bytes];
    for(int i = 0; i < userData.length; i++){
        [userDataTag appendFormat:@"%02x", *ptrr++ & 0xFF ];
    }
    [self addScanDataFiltering:tag rawData:pcEpc selectAnt:ant RSSI:rssi user:userDataTag];
}
//----------------------------------------------
#pragma mark - Private Method
//----------------------------------------------
-(void)showAlertViewIPs
{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@" * Found Device *"
                                                                   message:@"Please select device IP"
                                                            preferredStyle:UIAlertControllerStyleActionSheet];
    
    for(int i=0; i<[arrIPs count]; i++) {
        UIAlertAction* ip = [UIAlertAction actionWithTitle:[arrIPs objectAtIndex:i]
                                                     style:UIAlertActionStyleDefault
                                                   handler:^(UIAlertAction * action) {
                                                       self->selectIP =[self->arrIPs objectAtIndex:i];
                                                       [[ASRP31NSDK sharedInstance] connectServer:self->selectIP port:5000];}];
        [alert addAction:ip];
    }
    
    UIAlertAction *cancel = [UIAlertAction actionWithTitle:@"Cancel"
                                                     style:UIAlertActionStyleDefault
                                                   handler:nil];
    [alert addAction:cancel];
    
    UIBarButtonItem *button;
    alert.popoverPresentationController.barButtonItem = button;
    alert.popoverPresentationController.sourceView = self.view;
    [self presentViewController:alert animated:YES completion:nil];
}

-(void) enableUis:(BOOL)isEnable
{
    dispatch_async(dispatch_get_main_queue(),^{
        
        if(isEnable) {
            self.labelIP.text = @"Connected";
            self.labelIP.text = [NSString stringWithFormat:@"%@",self->selectIP ];
            [self.viewBottom setHidden:NO];
            [self.btnMore setEnabled:YES];
        } else {
            self.labelIP.text = @"Disconnected";
            [self.viewBottom setHidden:YES];
            [self.btnMore setEnabled:NO];
        }
    });
}

-(void)addScanDataFiltering:(NSString*)pcEpc  rawData:(NSData*)dataRaw selectAnt:(int)ant RSSI:(int)nRSSI user:(NSString *)user
{
    BOOL isNew = TRUE;
    NSString     *sAnt   = [NSString stringWithFormat:@"%d",ant];
    
    NSString *rssiValue     = @"";
    
    /* nRSSI == 0 mean RSSI value is none */
    if(nRSSI!=0)
        rssiValue =  [NSString stringWithFormat:@"%d",nRSSI];
    else
        rssiValue = @"";
    
    
    /* New data verification work */
    for(int i=0; i<tagData.arrTags.count;i++)
    {
        NSDictionary *dic    = [tagData.arrTags objectAtIndex:i];
        NSString     *oldPcEpc = [dic objectForKey:df_CELL_TAG_INFO];
        
        
        /* If it was previous data*/
        if ([oldPcEpc isEqualToString:pcEpc])
        {
            int nCount = [[dic objectForKey:df_CELL_TAG_COUNT] intValue]+1;
            NSString *strCount = [NSString stringWithFormat:@"%d",nCount];
            
            NSDictionary *inserDic  = [NSDictionary dictionaryWithObjectsAndKeys:
                                       pcEpc, df_CELL_TAG_INFO,
                                       strCount, df_CELL_TAG_COUNT,
                                       sAnt,df_CELL_TAG_ANT,
                                       rssiValue,df_CELL_TAG_RSSI,
                                       user,df_CELL_TAG_USER,
                                       dataRaw,df_CELL_TAG_RAW,nil];
            
            [tagData.arrTags replaceObjectAtIndex:i withObject:inserDic];
            isNew = FALSE;
            break;
        }
    }
    
    /* In the case of new data */
    if(isNew)
    {
        NSString *strCount = @"1";
        NSDictionary *inserDic  = [NSDictionary dictionaryWithObjectsAndKeys:
                                   pcEpc, df_CELL_TAG_INFO,
                                   strCount, df_CELL_TAG_COUNT,
                                   sAnt,df_CELL_TAG_ANT,
                                   dataRaw,df_CELL_TAG_RAW,nil];
        
        [tagData.arrTags addObject:inserDic];
        [self playBeepSound];
    }
    
    dispatch_async(dispatch_get_main_queue(),^{
        
        [self.tagViewController updateData];
        [self.labelTags setText:[NSString stringWithFormat:@"%d Tags",(int)[self->tagData.arrTags count]]];
        
    });
}

- (void)setUpSound
{
    NSURL *tapSound = [[NSBundle mainBundle] URLForResource:@"read" withExtension:@"caf"];
    CFURLRef soundFileURLRef = (__bridge CFURLRef)tapSound;
    AudioServicesCreateSystemSoundID (soundFileURLRef, &soundFileObject);
}

- (void)playBeepSound
{
    AudioServicesPlaySystemSound(soundFileObject);
}
@end
